var Structr = require('structr'),
EventEmitter = require('sk/core/events').EventEmitter,
Router = require('../../controller'),
Janitor = require('sk/core/garbage').Janitor;


var Message = Structr({
	
	/**
	 */

	'__construct': function(name, manager)
	{
		this._name = name;
		this._manager = manager;
	},

	/**
	 */

	'action': function(value)
	{
		this._action = value;
		return this;
	},


	/**
	 */

	'data': function(value)
	{
		this._data = value;
		return this;
	},

	/**
	 */

	'send': function(action, value)
	{
		if(action)
		{
			this.action(action);
			this.data(value);
		}

		var self = this;

		this._manager._connection._target.send(Structr.copy(this._buildMessage()), function(err, result)
		{
			if(err)
			{
				self.onError(err);
			}
		});

		return this;
	},

	/**
	 */

	'onError': function(e) { },

	/**
	 */

	'_buildMessage': function()
	{
		return { name: this._name, action: this._action, data: this._data };
	}
})

//back and forth communication
var Transaction = Message.extend({
	
	/**
	 */

	'override __construct': function(name, uid, manager)
	{
		this._super(name, manager);

		this._uid = uid;
		this._manager = manager;

		var em = this._em = new EventEmitter(),
		oldDispose = this._em.dispose,
		self = this;

		this._data = {};

		this._em.dispose = function()
		{
			this.disposed = true;
			oldDispose.call(em);
			self.dispose();
		}
	},

	/**
	 */

	'override _buildMessage': function()
	{
		var message = this._super();
		message.uid = this._uid;
		return message;
	},

	/**
	 */

	'response': function(message)
	{
		this._em.emit(message.action, message.data);
		return this;
	},

	/**
	 */

	'on': function(listen)
	{
		for(var type in listen)
		{
			this.on(type, listen[type])
		}

		return this;
	},

	/**
	 */

	'second on': function(type, listener)
	{
		this._em.addListener(type, listener);
		return this;
	},

	/**
	 */

	'register': function()
	{
		this._manager._addTransaction(this);	
		return this;
	},

	/**
	 */

	'onError': function(e)
	{
		this._em.emit('error', e);
	},

	/**
	 */

	'disposeOn': function(type)
	{
		var self =this;
		return this.on(type, function()
		{
			self.dispose();
		});
	},

	/**
	 */

	'dispose': function()
	{
		if(!this._em.disposed) this._em.dispose();

		this._manager.remove(this._uid);
	}
}); 


var CommunicationManager = Structr({

	/**
	 */

	'__construct': function(connection)
	{
		this._connection = connection;

		this._liveTransactions = {};

		this._connection._target.onMessage = this.getMethod('onMessage');
		this._em = new EventEmitter();
	},


	/**
	 */

	'message': function(name)
	{
		return new Message(name, this);
	},

	/**
	 */

	'on': function(listen)
	{
		for(var type in listen)
		{
			this._em.addListener(type, listen[type]);
		}	
	},

	/**
	 */

	'onMessage': function(message)
	{
		var trans = this._liveTransactions[ message.uid ];

		this._connection._hook.ignore(message.name, true);


		if(trans)
		{
			trans.response(message);
		}
		else
		{

			var msg;

			if(message.uid)
			{
				msg = this.request(message).action(message.action).data(message.data);
			}
			else
			{
				msg = this.message(message.name).action(message.action).data(message.data);
			}

			this._em.emit(msg._action, msg);
		}

		this._connection._hook.ignore(message.name, false);

		return false;
	},

	/**
	 */

	'onDisconnect': function(err)
	{
		for(var uid in this._liveTransactions)
		{
			var transaction = this._liveTransactions[uid];
			
			transaction.response({ action: 'error', data: 'unable to fullfill request' }).dispose();
		}	
	},

	
	/**
	 */
	
	'transaction': function(name)
	{
		return new Transaction(name, this._uid(), this).register();
	},

	/**
	 * response to transaction
	 */

	'request': function(message)
	{
		return new Transaction(message.name, message.uid, this);
	},

	/**
	 */
	
	'remove': function(uid)
	{
		var trans = this._liveTransactions[uid];
		delete this._liveTransactions[uid];
		return trans;
	},

	/**
	 */

	'_addTransaction': function(trans)
	{		
		this._liveTransactions[ trans._uid ] = trans;
	},


	/**
	 */

	'_uid': function()
	{
		var uid = new Date().getTime() + '.' + Math.round(Math.random() * 99999);
		while(this._liveTransactions[uid]) uid = this._uid();
		return uid;
	}
	
});


var RemoteInvoker = Structr({

	/**
	 */

	'__construct': function(hook)
	{
		this._janitor = new Janitor();
		this._hook = hook;
		this.remote = Math.random();

		var self = this;
		this._hook._router.on('push -pull hook', function(hook)
		{
			self._hook._transaction().send('hook', hook).dispose();
		});

		this.reset();
	},

	/**
	 */

	'reset': function()
	{
		this.dispose();

		this._janitor.addDisposable(this._virtualRouter = new Router());
		this.push = this._virtualRouter.getMethod('push')
		this.pull = this._virtualRouter.getMethod('pull');

		var self = this;
	},


	'dispose': function()
	{
		this._janitor.dispose();
	},

	/**
	 */

	'hook': function(type, channel)
	{
		switch(type)
		{
			case 'push': return this._hookPush(channel);
			case 'pull': return this._hookPull(channel);
			default: return null;
		}
	},

	/**
	 */

	 '_hookPull': function(channel)
	 {
	 	var self = this;

	 	this._janitor.addDisposable(this._virtualRouter.on('pull -stream ' + channel, function(request)
	 	{
	 		//get direct ref to methods
	 		this.wrap();
	 		

	 		self._hook._transaction(channel).
	 		send('pull', { hasNext: this.hasNext(), data: this.data } ).
	 		on(this).
	 		on({
	 			next: this.next,
	 		}).disposeOn('end');
	 	}));
	 },

	 '_hookPush': function(channel)
	 {
	 	var self = this;

	 	this._janitor.addDisposable(this._virtualRouter.on('push -stream ' + channel, function()
	 	{

	 		var trans = self._hook._transaction(channel).
	 		send('push', { hasNext: this.hasNext() }).
	 		on({
	 			next: this.next
	 		});

	 		this.pipe({
	 			write: function(chunk)
	 			{
	 				trans.send('write', chunk);
	 			},
	 			end: function()
	 			{
	 				trans.send('end').dispose();
	 			}
	 		})
	 	}));
	 }
});

var LocalInvoker = Structr({
	
	/**
	 */

	'__construct': function(connection)
	{
		this._con     = connection;
		this._router  = connection._router;
		this._janitor = new Janitor();
	},

	/**
	 */

	'hook': function(type, channel, ops)
	{
		var self = this,
		remoteMethod = this._con._remote.getMethod(type);

		if(!ops) ops = {};
		if(!ops.meta) ops.meta = {};

		ops.meta.stream = true;

		this._janitor.addDisposable(this._router.on(type + ' ' + channel, ops, function(localRequest)
		{
			if(self._con._hook.ignore(channel)) return;

			var _next = this.hasNext() ? function()
			{
				localRequest.next();
			} : null;


			remoteMethod(channel, localRequest.data, { meta: { stream: true }, _next: _next }, function(remoteRequest)
			{
				if(type == 'pull')
				{
					remoteRequest.pipe(localRequest);
				}
				else
				{
					localRequest.pipe(remoteRequest);
				}
			});
		}));
	},

	/**
	 */

	'reset': function()
	{
		this._janitor.dispose();
	},

	/**
	 */

	'dispose': function()
	{
		this._janitor.dispose();
	},

	/**
	 */

	'push': function(trans)
	{
		this._router.push(' -stream ' + trans._name, {}, { from: this._con._remote, _next: this._next(trans) }, function(stream)
		{
			//after push, there will be a series of writes. registering the transaction makes the local router own it ~ redirects to here.
			trans.register().
			on(stream.wrap()).
			disposeOn('end');
		});
	},

	/**
	 */

	'pull': function(trans)
	{

		this._router.pull(' -stream ' + trans._name, trans._data.data, { from: this._con._remote, _next: this._next(trans) }, function()
		{
			this.pipe({
				respond: function(response)
				{
					trans.send('respond',response);
				},
				write: function(chunk)
				{
					trans.send('write', chunk);
				},
				end: function(err)
				{
					trans.send('end');
					trans.dispose();
				}
			});
		});	
	},

	/**
	 */

	'_next': function(trans)
	{
		return (trans._data || {}).hasNext ? function()
		{
			trans.send('next');
		} : null;
	}
})


/**
 * brings remote, and local together.
 */

 var Connection = Structr({
 	
 	/**
 	 */

 	'__construct': function(target, hook)
 	{

 		//the target connection
 		this._target = target;

 		//the hook which handles connections - owns this one.
 		this._hook = hook;

 		//global router
 		this._router = hook._router;

 		//handles conversation between two servers
 		this._comm = new CommunicationManager(this);

 		//cleanes up routes after disconnect
 		this._janitor = new Janitor();

 		//handles local invocation
 		this._local = new LocalInvoker(this).wrap();

 		//handles remote invocation
 		this._remote = new RemoteInvoker(this);

 		this.janitor = new Janitor();

 		//listen for remote pulls / pushes
 		this._comm.on({
 			push: this._local.push,
 			pull: this._local.pull,
 			hook: this.getMethod('onHook'),
 		});

 		this._onClose();
 	},

 	/**
 	 */

 	'_onClose': function()
 	{
 		var self = this;

		this._target.onExit = function(err)
		{
			self._comm.onDisconnect();
			self.dispose();
		}
 	},


 	/**
 	 * hooks the connection the remote connection
 	 */

 	'onHook': function(trans)
 	{	
 		this._reset();


 		var self = this,
 		hook = trans._data;



 		for(var i = hook.channels.length; i--;)
		{
			try
			{
				self._listen(hook.channels[i], hook);
			}
			catch(e)
			{
				// console.log(e.stack)
			}
		}

 		this._connected();

 		return this;
 	},

 	/**
 	 */

 	'hook': function(type, path, ops)
 	{
 		this._remote.hook(type, path, ops);
 		this._local.hook(type, path, ops);
 	},

 	/**
 	 */
 	
 	'_connected': function()
 	{
 		if(this._dispatchedConnected) return;
 		this._dispatchedConnected = true;

 		this._router.push('hook/connection', null , { from: this._remote });
 	},

 	/**
 	 */


 	'_listen': function(channel, hook)
 	{
 		var ops = { meta: Structr.copy(channel.meta || {}) },
 		meta = ops.meta;

 		ops.meta.pull = ops.meta.rotate = undefined;

 		ops.meta.stream = ops.meta.hooked = ops.meta.overridable = ops.meta['public'] = 1;

 		meta.cluster = this._remote.cluster = hook.cluster;

 		this.hook('push', channel.path, ops);

 		var route = this._router.getRoute('pull ' + channel.path, ops);

 		if(route && route.route && route.route.meta)
 		{	
 			if(route.route.meta.hooked == undefined && !route.router._allowMultiple)
			{
				return console.warn('local channel %s exists. Cannot hook remote channel.', channel.path);
			}
 		}

 		this.hook('pull', channel.path, ops);
 	},

 	/**
 	 */

 	'_reset': function()
 	{
 		this._remote.reset();
 		this._local.reset();
 	},

 	/**
 	 */

 	'dispose': function()
 	{
 		this._remote.dispose();
 		this._local.dispose();
 		this.janitor.dispose();

 	},

 	/**
 	 */

 	'_response': function(message)
 	{
 		return this._comm.response(message);
 	},

 	/**
 	 */

 	'_transaction': function(name)
 	{
 		return this._comm.transaction(name); 
 	},

 	/**
 	 */

 	'_message': function(name)
 	{
 		return this._comm.message(name);
 	}
 });



 var Hook = Structr({
 	
 	/**
 	 */

 	'__construct': function(transport, router)
 	{
 		this._router = router;

 		var connections = this._connections = [],
 		self = this;
 		
 		//listen for when a client connects 
 		transport.connect(function(connection)
 		{
 			var con = new Connection(connection, self);
 			connections.push(con);
 			
 			con.janitor.addDisposable({
 				dispose: function()
 				{
 					var i = connections.indexOf(con);
 					if(i > -1) connections.splice(i, 1);
 				}
 			});

 		});
 	},

 	/** 
 	 * ignore channel
 	 */

 	'ignore': function(name, value)
	{
		if(!this._ignoring) this._ignoring = [];
		var i = this._ignoring.indexOf(name);

		if(value == undefined) return i > -1;

		if(!value && i > -1)
		{
			this._ignoring.splice(i, 1);
		}                             
		else
		if(value && i == -1)
		{
			this._ignoring.push(name);
		}
	},
 });

 module.exports = Hook;